"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const makeHtmlPlugin_1 = require("./makeHtmlPlugin");
const http_1 = __importDefault(require("http"));
const path_1 = require("path");
const dev_middleware_1 = __importDefault(require("nollup/lib/dev-middleware"));
const express_1 = __importDefault(require("express"));
const loadConfigFile_1 = __importDefault(require("rollup/dist/loadConfigFile"));
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('cypress:rollup-dev-server');
/**
 * Inject HMR runtime into each bundle, since Nollup
 * does not do this.
 */
function injectHmrPlugin() {
    return {
        name: 'MyPlugin',
        transform: (code) => {
            return {
                // inject HMR runtime
                // TODO: see if this ruins the source map
                code: `
          if (module) {
            module.hot.accept(() => {
              window.location.reload()
            })
          }

          ${code}
        `,
            };
        },
    };
}
function start(devServerOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('OBject', Object.keys(devServerOptions.options));
        const rollupConfigObj = typeof devServerOptions.rollupConfig === 'string'
            ? yield loadConfigFile_1.default(devServerOptions.rollupConfig).then((configResult) => configResult.options)
            : devServerOptions.rollupConfig;
        debug('Resolved rollup config options', rollupConfigObj);
        const config = devServerOptions.options.specs
            .map((spec) => {
            return Object.assign(Object.assign({}, rollupConfigObj), { input: spec.absolute, plugins: (rollupConfigObj.plugins || []).concat(injectHmrPlugin()) });
        });
        const { devServerPublicPathRoute, projectRoot, supportFile } = devServerOptions.options.config;
        const app = express_1.default();
        const server = http_1.default.createServer(app);
        const contentBase = path_1.resolve(__dirname, projectRoot);
        /* random port between 3000 and 23000 */
        const port = parseInt(((Math.random() * 20000) + 3000).toFixed(0), 10);
        const nollup = dev_middleware_1.default(app, config, {
            contentBase,
            port,
            publicPath: devServerPublicPathRoute,
            hot: true,
        }, server);
        app.use(nollup);
        makeHtmlPlugin_1.makeHtmlPlugin(projectRoot, supportFile, app, devServerPublicPathRoute);
        return {
            port,
            server,
        };
    });
}
exports.start = start;
